@@ -3,7 +3,7 @@
  * This file is the private include for the top_application
  * related functions.
  * 
- * Copyright (C) 2005  Nokia
+ * Copyright (C) 2005 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
+
 #ifndef OSSO_APPLICATION_TOP_
 # define OSSO_APPLICATION_TOP_
 
@@ -28,7 +29,4 @@ struct _osso_top{
     gpointer data;
 };
 
-DBusHandlerResult _top_handler(osso_context_t *osso, DBusMessage *msg,
-			       gpointer data);
-
 #endif /* OSSO_APPLICATION_TOP_ */
