@@ -2,7 +2,7 @@
  * @file osso-rpc.c
  * This file implements rpc calls from one application to another.
  * 
- * Copyright (C) 2005  Nokia
+ * Copyright (C) 2005 Nokia Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,13 +17,13 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
  */
 
-/* NOTICE: Keep in sync with values in hildon-navigator/windowmanager.c! */
-#define APP_LAUNCH_SIGNAL_INTERFACE "com.nokia.tasknav.signal"
-#define APP_LAUNCH_SIGNAL_PATH      "/com/nokia/tasknav/signal"
-#define APP_LAUNCH_BANNER_METHOD    "app_launch_banner"
+#define TASK_NAV_SERVICE                    "com.nokia.tasknav"
+/* NOTICE: Keep these in sync with values in hildon-navigator/windowmanager.c! */
+#define APP_LAUNCH_BANNER_METHOD_INTERFACE  "com.nokia.tasknav.app_launch_banner"
+#define APP_LAUNCH_BANNER_METHOD_PATH       "/com/nokia/tasknav/app_launch_banner"
+#define APP_LAUNCH_BANNER_METHOD            "app_launch_banner"
 
 
 #include "osso-internal.h"
@@ -74,11 +74,12 @@ osso_return_t osso_rpc_run(osso_context_t *osso, const gchar *service,
 
 
 /************************************************************************/
-osso_return_t _rpc_run(osso_context_t *osso, DBusConnection *conn,
-		       const gchar *service, const gchar *object_path,
-		       const gchar *interface, const gchar *method,
-		       osso_rpc_t *retval, int first_arg_type,
-		       va_list var_args)
+osso_return_t __attribute__ ((visibility("hidden")))
+_rpc_run(osso_context_t *osso, DBusConnection *conn,
+         const gchar *service, const gchar *object_path,
+         const gchar *interface, const gchar *method,
+         osso_rpc_t *retval, int first_arg_type,
+         va_list var_args)
 {
     DBusMessage *msg;
     dbus_bool_t b;
@@ -104,10 +105,11 @@ osso_return_t _rpc_run(osso_context_t *osso, DBusConnection *conn,
             dbus_connection_flush(conn);
             dbus_message_unref(msg);
 
-	    /* Signal TaskNavigator to show "launch banner" */
-	    msg = dbus_message_new_signal(
-			    APP_LAUNCH_SIGNAL_PATH,
-			    APP_LAUNCH_SIGNAL_INTERFACE,
+	    /* Tell TaskNavigator to show "launch banner" */
+	    msg = dbus_message_new_method_call(
+			    TASK_NAV_SERVICE,
+			    APP_LAUNCH_BANNER_METHOD_PATH,
+			    APP_LAUNCH_BANNER_METHOD_INTERFACE,
 			    APP_LAUNCH_BANNER_METHOD );
    
 	    if ( msg != NULL ) {
@@ -141,6 +143,8 @@ osso_return_t _rpc_run(osso_context_t *osso, DBusConnection *conn,
 	ret = dbus_connection_send_with_reply_and_block(conn, msg, 
 							osso->rpc_timeout,
 							&err);
+        dbus_message_unref(msg);
+
         if (!ret) {
             dprint("Error return: %s",err.message);
 	    retval->type = DBUS_TYPE_STRING;
@@ -167,10 +171,11 @@ osso_return_t _rpc_run(osso_context_t *osso, DBusConnection *conn,
 	    
 		dbus_message_unref(ret);
 	
-		/* Signal TaskNavigator to show "launch banner" */
-		msg = dbus_message_new_signal(
-				APP_LAUNCH_SIGNAL_PATH,
-				APP_LAUNCH_SIGNAL_INTERFACE,
+		/* Tell TaskNavigator to show "launch banner" */
+		msg = dbus_message_new_method_call(
+				TASK_NAV_SERVICE,
+				APP_LAUNCH_BANNER_METHOD_PATH,
+				APP_LAUNCH_BANNER_METHOD_INTERFACE,
 				APP_LAUNCH_BANNER_METHOD );
 
 		if ( msg != NULL ) {
@@ -232,35 +237,7 @@ osso_return_t osso_rpc_run_with_defaults(osso_context_t *osso,
 }
 
 /************************************************************************/
-osso_return_t osso_rpc_async_run(osso_context_t *osso,
-				 const gchar *service,
-				 const gchar *object_path,
-				 const gchar *interface,
-				 const gchar *method,
-				 osso_rpc_async_f *async_cb,
-				 gpointer data,
-				 int argument_type, ...)
-{
-    va_list arg_list;
-    osso_return_t ret;
-   
-	
-    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
-	return OSSO_INVALID;
-
-    va_start(arg_list, argument_type);
-
-    ret = _rpc_async_run(osso, service, object_path, interface, method,
-			 async_cb, data, argument_type, arg_list);
-    
-    va_end(arg_list);
-    
-    return ret;
-}
-
-/************************************************************************/
-osso_return_t _rpc_async_run(osso_context_t *osso,
+static osso_return_t _rpc_async_run(osso_context_t *osso,
 			     const gchar *service,
 			     const gchar *object_path,
 			     const gchar *interface,
@@ -295,7 +272,7 @@ osso_return_t _rpc_async_run(osso_context_t *osso,
 	b = dbus_connection_send(osso->conn, msg, NULL);
     }
     else {
-	rpc = (_osso_rpc_async_t *)malloc(sizeof(_osso_rpc_async_t));
+	rpc = (_osso_rpc_async_t *)calloc(1, sizeof(_osso_rpc_async_t));
 	if(rpc == NULL) return OSSO_ERROR;
 	
 	rpc->func = async_cb;
@@ -321,10 +298,11 @@ osso_return_t _rpc_async_run(osso_context_t *osso,
 	dbus_connection_flush(osso->conn);
 	dbus_message_unref(msg);
 
-	/* Signal TaskNavigator to show "launch banner" */
-	msg = dbus_message_new_signal(
-			APP_LAUNCH_SIGNAL_PATH,
-			APP_LAUNCH_SIGNAL_INTERFACE,
+	/* Tell TaskNavigator to show "launch banner" */
+	msg = dbus_message_new_method_call(
+			TASK_NAV_SERVICE,
+			APP_LAUNCH_BANNER_METHOD_PATH,
+			APP_LAUNCH_BANNER_METHOD_INTERFACE,
 			APP_LAUNCH_BANNER_METHOD );
 
 	if ( msg != NULL ) {
@@ -351,6 +329,34 @@ osso_return_t _rpc_async_run(osso_context_t *osso,
     }
 }
 
+/************************************************************************/
+osso_return_t osso_rpc_async_run(osso_context_t *osso,
+				 const gchar *service,
+				 const gchar *object_path,
+				 const gchar *interface,
+				 const gchar *method,
+				 osso_rpc_async_f *async_cb,
+				 gpointer data,
+				 int argument_type, ...)
+{
+    va_list arg_list;
+    osso_return_t ret;
+   
+	
+    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
+	(interface == NULL) || (method == NULL))
+	return OSSO_INVALID;
+
+    va_start(arg_list, argument_type);
+
+    ret = _rpc_async_run(osso, service, object_path, interface, method,
+			 async_cb, data, argument_type, arg_list);
+    
+    va_end(arg_list);
+    
+    return ret;
+}
+
 /************************************************************************/
 osso_return_t osso_rpc_async_run_with_defaults(osso_context_t *osso,
 					       const gchar *application,
@@ -382,20 +388,7 @@ osso_return_t osso_rpc_async_run_with_defaults(osso_context_t *osso,
 }
 
 /************************************************************************/
-
-osso_return_t osso_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
-		       const gchar *object_path, const gchar *interface,
-		       osso_rpc_cb_f *cb, gpointer data)
-{
-    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (cb == NULL))
-	return OSSO_INVALID;
-    return _rpc_set_cb_f(osso, service, object_path,
-			 interface,cb, data, FALSE);
-}
-/************************************************************************/
-
-osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
+static osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
 			    const gchar *object_path,
 			    const gchar *interface,
 			    osso_rpc_cb_f *cb, gpointer data,
@@ -411,15 +404,18 @@ osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
 	   " on the %sbus", service, object_path, interface,
 	   use_system_bus?"SYSTEM":"SESSION");
     
-    rpc = (_osso_rpc_t *)malloc(sizeof(_osso_rpc_t));
-    if(rpc == NULL) return OSSO_ERROR;
+    rpc = (_osso_rpc_t *)calloc(1, sizeof(_osso_rpc_t));
+    if (rpc == NULL) {
+        ULOG_ERR_F("calloc failed");
+        return OSSO_ERROR;
+    }
     
     dbus_error_init(&err);
 
     dprint("acquiring service '%s'", service);
     i = dbus_bus_acquire_service(use_system_bus?osso->sys_conn:osso->conn,
 				 service, 0, &err);
-#ifdef DEBUG
+#ifdef LIBOSSO_DEBUG
     if(i <= 0) {
 	if(i == 0) {
 	    dprint("Unable to acquire service '%s': Invalid parameters\n", 
@@ -457,6 +453,29 @@ osso_return_t _rpc_set_cb_f(osso_context_t *osso, const gchar *service,
     return OSSO_OK;
 }
 
+/************************************************************************/
+osso_return_t _test_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
+                                 const gchar *object_path,
+                                 const gchar *interface,
+                                 osso_rpc_cb_f *cb, gpointer data,
+                                 gboolean use_system_bus)
+{
+    return _rpc_set_cb_f(osso, service, object_path, interface,
+                         cb, data, use_system_bus);
+}
+
+/************************************************************************/
+osso_return_t osso_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
+		       const gchar *object_path, const gchar *interface,
+		       osso_rpc_cb_f *cb, gpointer data)
+{
+    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
+	(interface == NULL) || (cb == NULL))
+	return OSSO_INVALID;
+    return _rpc_set_cb_f(osso, service, object_path,
+			 interface,cb, data, FALSE);
+}
+
 /************************************************************************/
 osso_return_t osso_rpc_set_default_cb_f(osso_context_t *osso,
 					osso_rpc_cb_f *cb,
@@ -468,15 +487,17 @@ osso_return_t osso_rpc_set_default_cb_f(osso_context_t *osso,
     if( (osso == NULL) || (cb == NULL) )
 	return OSSO_INVALID;
 
-    rpc = (_osso_rpc_t *)malloc(sizeof(_osso_rpc_t));
-    if(rpc == NULL)
+    rpc = (_osso_rpc_t *)calloc(1, sizeof(_osso_rpc_t));
+    if (rpc == NULL) {
+        ULOG_ERR_F("calloc failed");
 	return OSSO_ERROR;
+    }
     
     g_snprintf(interface, 255, "%s.%s", OSSO_BUS_ROOT, osso->application);    
     rpc->func = cb;
     rpc->data = data;
     
-    _msg_handler_set_cb_f(osso, interface, _rpc_handler,
+    _msg_handler_set_cb_f_free_data(osso, interface, _rpc_handler,
 			  (gpointer)rpc, TRUE);
     return OSSO_OK;
 }
@@ -732,6 +753,8 @@ static void _append_arg(DBusMessage *msg, osso_rpc_t *arg)
 
 static void _get_arg(DBusMessageIter *iter, osso_rpc_t *retval)
 {
+    char *str;
+
     dprint("");
 
     retval->type = dbus_message_iter_get_arg_type(iter);
@@ -753,8 +776,9 @@ static void _get_arg(DBusMessageIter *iter, osso_rpc_t *retval)
 	dprint("got DOUBLE:%f",retval->value.d);
 	break;
       case DBUS_TYPE_STRING:
-	retval->value.s = 
-	    g_strdup(dbus_message_iter_get_string(iter));
+	str = dbus_message_iter_get_string (iter);
+	retval->value.s = g_strdup (str);
+	dbus_free (str);
 	if(retval->value.s == NULL) {
 	    retval->type = DBUS_TYPE_INVALID;
 	}
@@ -780,9 +804,7 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
     osso_rpc_t retval;
     int type;
     rpc = (_osso_rpc_async_t *)data;
-    DLOG_OPEN("libosso");
-    DLOG_INFO_F("At msg return handler");
-    LOG_CLOSE();
+    ULOG_INFO_F("At msg return handler");
     msg = dbus_pending_call_get_reply(pending);
     if(msg == NULL) {
 	g_free(rpc->interface);
@@ -807,10 +829,9 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
 	DBusError err;
 	dbus_error_init(&err);
 	dbus_set_error_from_message(&err, msg);
-	retval.type = DBUS_TYPE_INVALID;
-	retval.value.s = g_strdup(err.message);
 	dbus_error_free(&err);
     }
+
     dbus_message_unref(msg);
     dprint("rpc = %p",rpc);
     dprint("rpc->func = %p",rpc->func);
